package de.bsvrz.sys.funclib.bitctrl.modell.util.geolib;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.sys.funclib.bitctrl.geolib.UTMKoordinate;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Polygon;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Punkt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmgeoreferenzierungglobal.objekte.Linie;
import de.bsvrz.sys.funclib.bitctrl.modell.tmgeoreferenzierungglobal.objekte.Punkt;

/**
 * Macht die geolib aus funclib.bitctrl für die Modell-Objekte nutzbar. Diese
 * Klasse arbeitet nur mit den UTM-Koordinaten.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 * @see WGS84Util
 */
public final class UTMUtil {

	/**
	 * Bestimmt die UTM-Koordinaten eines Punkts. Erweitert ein Objekt sowohl
	 * <code>PunktXY</code> als auch <code>PunktLiegtAufLinienObjekt</code>,
	 * wird zuerst versucht die Koordinaten über
	 * <code>PunktLiegtAufLinienObjekt</code> zu bestimmen, ist das nicht
	 * möglich, wird versucht sie über <code>PunktXY</code> zu bestimmen.
	 * 
	 * @param punkt
	 *            der Punkt, dessen Koordinaten gesucht sind.
	 * @return die Koordinaten des Punkts.
	 */
	public static UTMKoordinate konvertiere(final Punkt punkt) {
		return WGS84Util.konvertiereIntern(punkt).toUTMKoordinate();
	}

	/**
	 * Bestimmt die Liste der Koordinaten für eine Linie.
	 * 
	 * @param linie
	 *            die Linie, deren Koordinaten gesucht sind.
	 * @return die Liste der Koordinaten der Linie.
	 */
	public static List<UTMKoordinate> konvertiere(final Linie linie) {
		final WGS84Polygon wgs84 = WGS84Util.konvertiereIntern(linie);
		final List<UTMKoordinate> result = new ArrayList<>();
		for (final WGS84Punkt p : wgs84.getKoordinaten()) {
			result.add(p.toUTMKoordinate());
		}
		return result;
	}

	private UTMUtil() {
		// utility class
	}

}
